      subroutine overtn
c
c ---  diagnose meridional overturning and heat fluxes in basin model
c
      include 'dimensions.h'
      include 'common_blocks.h'
c
      common/linepr/lp
c
      character text*24,flnm*10
      common/work/flux(kdm,idm),heatfl(idm),mask(kdm,idm),thsurf(idm),
     .            zonavt(kdm,idm),zonavf(kdm,idm),hfxzon(idm),
     .            work(kdm,idm)
c
c --- integrate meridional mass fluxes in zonal direction (northward = plus)
      do 1 k=1,kk
      flux(k,1 )=0.
      flux(k,ii)=0.
      do 1 i=2,ii1
      flux(k,i)=0.
      do 1 l=1,jsu(i)
      do 1 j=jfu(i,l),jlu(i,l)
 1    flux(k,i)=flux(k,i)-uflx(i,j,k)
c
c --- integrate meridional heat fluxes vertically and in zonal direction
      heatfl( 1)=0.
      heatfl(ii)=0.
      hfxzon( 1)=0.
      hfxzon(ii)=0.
c
      do 12 i=2,ii1
      heatfl(i)=0.
      hfxzon(i)=0.
c
      do 13 k=1,kk
      num=0
      zonavt(k,i)=0.
      zonavf(k,i)=0.
c
      do 11 l=1,jsu(i)
      do 11 j=jfu(i,l),jlu(i,l)
      num=num+1
      zonavt(k,i)=zonavt(k,i)+(temp(i,j,k)+temp(i-1,j,k))
      zonavf(k,i)=zonavf(k,i)+uflx(i,j,k)
 11   heatfl(i)=heatfl(i)+(temp(i,j,k)+temp(i-1,j,k))*uflx(i,j,k)
c
 13   hfxzon(i)=hfxzon(i)+zonavt(k,i)*zonavf(k,i)/max(float(num),1.)
      hfxzon(i)=-.5*hfxzon(i)*spcifh/g * 1.e-15
 12   heatfl(i)=-.5*heatfl(i)*spcifh/g * 1.e-15
c
c --- add mass fluxes vertically to produce stream function values
      do 2 k=2,kk
      do 2 i=1,ii1
 2    flux(k,i)=flux(k,i)+flux(k-1,i)
c
c --- find zonally averaged mixed-layer density
      do 7 i=1,ii1
      thsurf(i)=0.
      thk=0.
      do 8 l=1,jsp(i)
      do 8 j=jfp(i,l),jlp(i,l)
      thsurf(i)=thsurf(i)+thmix(i,j,1)*dp(i,j,1)
 8    thk=thk+dp(i,j,1)*float(ip(i,j))
 7    thsurf(i)=thsurf(i)/thk
c
c --- zero out portions of (i,k) space located above sea surface
      do 6 k=1,kk-1
      do 6 i=1,ii1
      if (theta(k+1).lt.thsurf(i)) flux(k,i)=0.
 6    continue
c
c --- contract mass and heat flux arrays in i direction
      imx=ii
      do 4 it=1,alog(float(ii1))/alog(2.)-4
      imx=(imx+1)/2-1
      do 4 i=1,imx
      hfxzon(i)=.5*hfxzon(2*i)+.25*(hfxzon(2*i-1)+hfxzon(2*i+1))
      heatfl(i)=.5*heatfl(2*i)+.25*(heatfl(2*i-1)+heatfl(2*i+1))
      do 4 k=1,kk
 4    flux(k,i)=.5*flux(k,2*i)+.25*(flux(k,2*i-1)+flux(k,2*i+1))
c
      write (lp,'('' northward heat flux (petawatts):'',/(11f7.3))')
     .   (heatfl(i),i=1,imx)
      write (lp,'('' merid.overturning component:'',/(11f7.3))')
     .   (hfxzon(i),i=1,imx)
c
c --- find scale factor for printing mass flux streamfunction
      scale=0.
      do 3 i=1,imx
      do 3 k=1,kk
      mask(k,i)=1
 3    scale=max(scale,abs(flux(k,i)))
      scale=10.**(int(alog10(scale))-2)
c
      write (text,'(''merid.overturn.('',1p,e7.1,'')'')') scale
      call prtmsk(mask,flux,work,kdm,kdm,imx,0.,1./scale,text)
c
      call zebra(flux,kdm,kdm,imx)
c
c --- save everything in a special file
      no=19
      write (flnm,'(i10)') int(time+.001E+00)+1000000
      flnm(1:5)='ovtn.'
      open (unit=no,file=flnm,status='unknown')
c
      keep=lp
      lp=no
      write (lp,'(1x,a10,'' - + - + - + - + - + - + - + - +'')') flnm
      write (lp,'('' northward heat flux (petawatts):'',/(11f7.3))')
     .   (heatfl(i),i=1,imx)
      write (lp,'('' merid.overturning component:'',/(11f7.3))')
     .   (hfxzon(i),i=1,imx)
      call prtmsk(mask,flux,work,kdm,kdm,imx,0.,1./scale,text)
      close (unit=no)
      lp=keep
c
      return
      end
