      subroutine thermf(m,n,mm,nn,k1m,k1n)
c
c --- version 2.6 -- cyclic in j
      implicit none
c
      include 'dimensions.h'
      include 'common_blocks.h'
c
      common/linepr/lp
c
      real qsatur,thknss,radfl,wind,airt,vpmx,prcp,exchng,evap,emnp,
     .     watts,emnsp
      integer lp
c
      include 'stmt_functions.h'
c
c --- saturation specific humidity (lowe, j.appl.met., 16, 100-103, 1976)
      qsatur(t)=.622e-3*(6.107799961e+00+t*(4.436518521e-01
     .               +t*(1.428945805e-02+t*(2.650648471e-04
     .               +t*(3.031240396e-06+t*(2.034080948e-08
     .               +t* 6.136820929e-11))))))
c
c --- ----------------------------
c --- thermal forcing at sidewalls
c --- ----------------------------
c
      if (relax) then
c
c --- (a) northern wall (nbdy = number of grid rows affected)
c
      do 54 i=1,nbdy
      do 54 l=1,jsp(i)
c
c --- interior interfaces:
      do 53 k=3,kk
      do 53 j=jfp(i,l),jlp(i,l)
          p(i,j,k)=p(i,j,k)+delt1*rmu(i)*
     .   (pwall(i,j,k,l0)*w0+pwall(i,j,k,l1)*w1
     .   +pwall(i,j,k,l2)*w2+pwall(i,j,k,l3)*w3
     .  - p(i,j,k))
 53   continue
c
c --- mixed-layer temperature:
      do 59 j=jfp(i,l),jlp(i,l)
        temp(i,j,k1n)=temp(i,j,k1n)+delt1*rmu(i)*
     .  (twall(i,j,l0)*w0+twall(i,j,l1)*w1
     .  +twall(i,j,l2)*w2+twall(i,j,l3)*w3
     . - temp(i,j,k1n))
 59   continue
c
c --- salinity (all layers):
      do 54 k=kk,2,-1
      kn=k+nn
      do 54 j=jfp(i,l),jlp(i,l)
      p(i,j,k)=amin1(amax1(p(i,j,k),p(i,j,2)),p(i,j,kk+1))
      dp(i,j,kn)=p(i,j,k+1)-p(i,j,k)
c
 54   saln(i,j,kn)=saln(i,j,kn)+delt1*rmu(i)*
     .   (swall(i,j,k,ls0)*ws0+swall(i,j,k,ls1)*ws1
     .   +swall(i,j,k,ls2)*ws2+swall(i,j,k,ls3)*ws3
     .   - saln(i,j,kn))
c
c --- (b) southern wall (nbdy = number of grid rows affected)
c
      do 64 i=ii-nbdy,ii1
      do 64 l=1,jsp(i)
c
c --- interior interfaces:
      do 63 k=3,kk
      do 63 j=jfp(i,l),jlp(i,l)
          p(i,j,k)=p(i,j,k)+delt1*rmu(ii-i)*
     .   (pwall(i,j,k,l0)*w0+pwall(i,j,k,l1)*w1
     .   +pwall(i,j,k,l2)*w2+pwall(i,j,k,l3)*w3
     .  - p(i,j,k))
 63   continue
c
c --- mixed-layer temperature:
      do 58 j=jfp(i,l),jlp(i,l)
        temp(i,j,k1n)=temp(i,j,k1n)+delt1*rmu(ii-i)*
     .  (twall(i,j,l0)*w0+twall(i,j,l1)*w1
     .  +twall(i,j,l2)*w2+twall(i,j,l3)*w3
     . - temp(i,j,k1n))
 58   continue
c
c --- salinity (all layers):
      do 64 k=kk,2,-1
      kn=k+nn
      do 64 j=jfp(i,l),jlp(i,l)
      p(i,j,k)=amin1(amax1(p(i,j,k),p(i,j,2)),p(i,j,kk+1))
      dp(i,j,kn)=p(i,j,k+1)-p(i,j,k)
c
 64   saln(i,j,kn)=saln(i,j,kn)+delt1*rmu(ii-i)*
     .   (swall(i,j,k,ls0)*ws0+swall(i,j,k,ls1)*ws1
     .   +swall(i,j,k,ls2)*ws2+swall(i,j,k,ls3)*ws3
     .   - saln(i,j,kn))
c
      endif              !  relax
c
c --- --------------------------------
c --- thermal forcing of ocean surface
c --- --------------------------------
c
      watts=0.
      emnsp=0.
c
      do 85 j=1,jj
      do 85 l=1,isp(j)
c
      do 85 i=ifp(j,l),ilp(j,l)
      thknss=dp(i,j,k1n)
      radfl=radflx(i,j,l0)*w0+radflx(i,j,l1)*w1
     .     +radflx(i,j,l2)*w2+radflx(i,j,l3)*w3
      wind=wndspd(i,j,l0)*w0+wndspd(i,j,l1)*w1
     .    +wndspd(i,j,l2)*w2+wndspd(i,j,l3)*w3
      ustar(i,j)=sqrt(thref*cd*airdns)*wind
      airt=airtmp(i,j,l0)*w0+airtmp(i,j,l1)*w1
     .    +airtmp(i,j,l2)*w2+airtmp(i,j,l3)*w3
c --- vpmx = water vapor mixing ratio (g/g)
      vpmx=vapmix(i,j,l0)*w0+vapmix(i,j,l1)*w1
     .    +vapmix(i,j,l2)*w2+vapmix(i,j,l3)*w3
c --- prcp = precipitation (cm/sec; < 0 since rain falls in -z direction)
      prcp=precip(i,j,l0)*w0+precip(i,j,l1)*w1
     .    +precip(i,j,l2)*w2+precip(i,j,l3)*w3
c
      exchng=ct*airdns*wind
c --- evap = evaporation (w/cm**2)
      evap=exchng*evaplh*max(0.,qsatur(temp(i,j,k1n))-vpmx)
c --- surflx = thermal energy flux (w/cm**2) in +z direction
      surflx(i,j)=exchng*csubp*(temp(i,j,k1n)-airt) + evap + radfl
c --- emnp = evaporation minus precipitation (cm/sec)
      emnp=evap*thref/evaplh + prcp
c --- salflx = salt flux (10**-3 g/cm**2/sec) in +z direction
      salflx(i,j)=-35.0*emnp/thref
c
c --- n o t e : t/s changes due to surflx/salflx are computed in mxlayr routine
c
c --- buoyfl = buoyancy flux, w_prime_buoyancy_prime_bar (cm**2/sec**3)
c --- note: surface density increases (column is destabilized) if buoyfl > 0
      buoyfl(i,j)=
     .g*(dsigds(temp(i,j,k1n),saln(i,j,k1n))*saln(i,j,k1n)*emnp
     .  -dsigdt(temp(i,j,k1n),saln(i,j,k1n))*surflx(i,j)*thref/spcifh)
c
cdiag if (i.eq.itest.and.j.eq.jtest) write (lp,'(i9,2i4,''  radfl    '',
cdiag.''wind     airt      sst     vpmx    surflx    evap     prcp   '',
cdiag.''  emnp    buoyfl''/16x,1p10e9.2)') nstep,i,j,radfl,wind,airt,
cdiag.temp(i,j,k1n),vpmx,surflx(i,j),evap,prcp,emnp,buoyfl(i,j)
c
      watts=watts+surflx(i,j)*scp2(i)
      watcum=watcum+surflx(i,j)*scp2(i)
      emnsp=emnsp+emnp*scp2(i)
      empcum=empcum+emnp*scp2(i)
 85   continue
c
      if (diagno) then
      write (lp,'(i9,'' energy residual (w/m**2)'',4p2f9.2)') nstep,
     . watts/area,watcum/(area*(nstep-nstep1))
      write (lp,'(i9,'' e - p residual (mm/year)'',4p2f9.2)') nstep,
     . emnsp/area*864.*36.,empcum/(area*(nstep-nstep1))*864.*36.
      end if
c
      return
      end
